@@ -6,7 +6,8 @@ use super::backend::{
 use crate::{
     codegen::backend::{DeclInfo, Keywords, Names, RelativeNamespace},
     intermediate_language::types::{
-        AbsolutePath, DeclarationKind, Declarations, NamespaceIndex, SimpleType, Type, TypeKind,
+        AbsolutePath, DeclarationIndex, DeclarationKind, Declarations, NamespaceIndex, SimpleType,
+        Type, TypeKind,
     },
 };
 
@@ -227,10 +228,10 @@ fn translate_type_index<'a, B: ?Sized + Backend>(
     translation_context: &DeclarationTranslationContext<'a, '_, B>,
     declarations: &'a Declarations,
     full_translated_decls: &'a VecMap<BackendDeclaration<B>>,
-    index: usize,
+    index: DeclarationIndex,
     current_namespace_path: &AbsolutePath,
 ) -> ResolvedType<'a, B> {
-    let (path, decl) = &translation_context.translated_decls[index];
+    let (path, decl) = &translation_context.translated_decls[index.0];
     let relative_path: RelativeNamespace<B> = RelativeNamespace::new(
         current_namespace_path,
         &path.clone_pop(),
@@ -246,7 +247,7 @@ fn translate_type_index<'a, B: ?Sized + Backend>(
         }
         DeclInfo::Enum(translated_decl, decl) => {
             if let BackendDeclaration::Enum(BackendEnum { variants, .. }) =
-                full_translated_decls.get(index).unwrap()
+                full_translated_decls.get(index.0).unwrap()
             {
                 ResolvedType::Enum(decl, translated_decl, relative_path, variants)
             } else {
@@ -275,7 +276,7 @@ fn translate_type<'a, B: ?Sized + Backend>(
             translation_context,
             declarations,
             full_translated_decls,
-            index.0,
+            *index,
             current_namespace_path,
         ),
         TypeKind::SimpleType(type_) => translate_simple_type(
@@ -318,7 +319,7 @@ fn translate_simple_type<'a, B: ?Sized + Backend>(
             translation_context,
             declarations,
             full_translated_decls,
-            index.0,
+            *index,
             current_namespace_path,
         ),
         SimpleType::Bool => ResolvedType::Bool,
@@ -390,7 +391,8 @@ pub fn run_backend<B: ?Sized + Backend>(
         .declarations
         .iter()
         .zip(&mut declaration_names)
-        .map(|((decl_name, decl), declaration_names)| {
+        .enumerate()
+        .map(|(decl_id, ((decl_name, decl), declaration_names))| {
             let namespace_names = &mut namespace_names[decl.namespace_id.0];
             let decl: DeclInfo<B> = match &decl.kind {
                 DeclarationKind::Table(decl) => DeclInfo::Table(
@@ -401,6 +403,7 @@ pub fn run_backend<B: ?Sized + Backend>(
                             declaration_names,
                         },
                         &translated_namespaces,
+                        DeclarationIndex(decl_id),
                         decl_name,
                         decl,
                     ),
@@ -414,6 +417,7 @@ pub fn run_backend<B: ?Sized + Backend>(
                             declaration_names,
                         },
                         &translated_namespaces,
+                        DeclarationIndex(decl_id),
                         decl_name,
                         decl,
                     ),
@@ -427,6 +431,7 @@ pub fn run_backend<B: ?Sized + Backend>(
                             declaration_names,
                         },
                         &translated_namespaces,
+                        DeclarationIndex(decl_id),
                         decl_name,
                         decl,
                     ),
@@ -440,6 +445,7 @@ pub fn run_backend<B: ?Sized + Backend>(
                             declaration_names,
                         },
                         &translated_namespaces,
+                        DeclarationIndex(decl_id),
                         decl_name,
                         decl,
                     ),
@@ -453,6 +459,7 @@ pub fn run_backend<B: ?Sized + Backend>(
                             declaration_names,
                         },
                         &translated_namespaces,
+                        DeclarationIndex(decl_id),
                         decl_name,
                         decl,
                     ),
