@@ -1,4 +1,4 @@
-use std::fmt::Display;
+use std::{collections::BTreeSet, fmt::Display};
 
 use codespan::{FileId, Span};
 use indexmap::IndexMap;
@@ -10,21 +10,27 @@ pub struct AbsolutePath(pub Vec<String>);
 
 impl AbsolutePath {
     pub const ROOT_PATH: Self = Self(Vec::new());
+
     pub fn push<S: Into<String>>(&mut self, value: S) {
         self.0.push(value.into())
     }
+
+    pub fn pop(&mut self) -> Option<String> {
+        self.0.pop()
+    }
+
+    #[must_use]
     pub fn clone_push<S: AsRef<str>>(&self, value: S) -> Self {
         let mut r = self.clone();
         r.push(value.as_ref());
         r
     }
+
+    #[must_use]
     pub fn clone_pop(&self) -> Self {
         assert!(!self.0.is_empty());
         Self(self.0[..self.0.len() - 1].to_vec())
     }
-    pub fn pop(&mut self) -> Option<String> {
-        self.0.pop()
-    }
 }
 
 impl std::fmt::Display for AbsolutePath {
@@ -47,12 +53,18 @@ pub struct RelativePath {
     pub remaining: Vec<String>,
 }
 
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub struct DeclarationIndex(pub usize);
 impl DeclarationIndex {
     pub const INVALID: DeclarationIndex = DeclarationIndex(usize::MAX);
 }
 
+impl Display for DeclarationIndex {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self.0)
+    }
+}
+
 impl<'a> From<&'a DeclarationIndex> for DeclarationIndex {
     fn from(decl: &'a DeclarationIndex) -> Self {
         *decl
@@ -87,6 +99,8 @@ impl<'a, 'b> From<&'b &'a NamespaceIndex> for NamespaceIndex {
 pub struct Declarations {
     pub namespaces: IndexMap<AbsolutePath, Namespace>,
     pub declarations: IndexMap<AbsolutePath, Declaration>,
+    pub children: Vec<Vec<DeclarationIndex>>,
+    pub parents: Vec<Vec<DeclarationIndex>>,
 }
 
 impl Declarations {
@@ -94,9 +108,31 @@ impl Declarations {
         namespaces: IndexMap<AbsolutePath, Namespace>,
         declarations: IndexMap<AbsolutePath, Declaration>,
     ) -> Self {
+        let children = declarations
+            .values()
+            .map(|decl| match &decl.kind {
+                DeclarationKind::Table(decl) => decl.children(),
+                DeclarationKind::Struct(decl) => decl.children(),
+                DeclarationKind::Enum(_) => Vec::new(),
+                DeclarationKind::Union(decl) => decl.children(),
+                DeclarationKind::RpcService(decl) => decl.children(),
+            })
+            .collect::<Vec<_>>();
+
+        let mut parents = (0..declarations.len())
+            .map(|_| Vec::new())
+            .collect::<Vec<_>>();
+        for (parent_decl_id, children) in children.iter().enumerate() {
+            for child_decl_id in children {
+                parents[child_decl_id.0].push(DeclarationIndex(parent_decl_id));
+            }
+        }
+
         Self {
             namespaces,
             declarations,
+            children,
+            parents,
         }
     }
 
@@ -125,6 +161,8 @@ impl Declarations {
 
 #[derive(Debug, Default)]
 pub struct Namespace {
+    /// The span is where the namespace path is defined
+    pub spans: Vec<(FileId, Option<Span>)>,
     pub child_namespaces: IndexMap<String, NamespaceIndex>,
     pub declaration_ids: IndexMap<String, DeclarationIndex>,
 }
@@ -242,6 +280,21 @@ impl TypeKind {
     pub fn is_enum(&self) -> bool {
         matches!(self, &TypeKind::SimpleType(SimpleType::Enum(..)))
     }
+
+    fn add_children(&self, children: &mut BTreeSet<DeclarationIndex>) {
+        match self {
+            TypeKind::Table(decl_id) | TypeKind::Union(decl_id) => {
+                children.insert(*decl_id);
+            }
+            TypeKind::Vector(type_) | TypeKind::Array(type_, _) => {
+                type_.kind.add_children(children)
+            }
+            TypeKind::SimpleType(kind) => {
+                kind.add_children(children);
+            }
+            TypeKind::String => (),
+        }
+    }
 }
 
 #[derive(Debug)]
@@ -273,6 +326,15 @@ impl SimpleType {
             | SimpleType::Float(_) => true,
         }
     }
+
+    fn add_children(&self, children: &mut BTreeSet<DeclarationIndex>) {
+        match self {
+            SimpleType::Struct(decl_id) | SimpleType::Enum(decl_id) => {
+                children.insert(*decl_id);
+            }
+            SimpleType::Bool | SimpleType::Integer(_) | SimpleType::Float(_) => (),
+        }
+    }
 }
 
 #[derive(Clone, Debug)]
@@ -316,6 +378,31 @@ impl IntegerLiteral {
     }
 }
 
+impl Display for Literal {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Literal::Bool(v) => write!(f, "{}", v),
+            Literal::String(v) => write!(f, "{}", v),
+            Literal::Int(v) => write!(f, "{}", v),
+            Literal::Float(v) => write!(f, "{}", v),
+            Literal::Array(vs) | Literal::Vector(vs) => {
+                write!(f, "[")?;
+                let mut first = true;
+                for v in vs {
+                    if !first {
+                        write!(f, ", {}", v)?;
+                    } else {
+                        first = false;
+                        write!(f, "{}", v)?;
+                    }
+                }
+                write!(f, "]")
+            }
+            Literal::EnumTag { value, .. } => write!(f, "{}", value),
+        }
+    }
+}
+
 impl Display for IntegerLiteral {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -345,3 +432,43 @@ impl Display for FloatLiteral {
         }
     }
 }
+impl Table {
+    fn children(&self) -> Vec<DeclarationIndex> {
+        let mut children = BTreeSet::new();
+        for field in self.fields.values() {
+            field.type_.kind.add_children(&mut children);
+        }
+        children.into_iter().collect()
+    }
+}
+
+impl Struct {
+    fn children(&self) -> Vec<DeclarationIndex> {
+        let mut children = BTreeSet::new();
+        for field in self.fields.values() {
+            field.type_.add_children(&mut children);
+        }
+        children.into_iter().collect()
+    }
+}
+
+impl Union {
+    fn children(&self) -> Vec<DeclarationIndex> {
+        let mut children = BTreeSet::new();
+        for variant in self.variants.values() {
+            variant.type_.kind.add_children(&mut children);
+        }
+        children.into_iter().collect()
+    }
+}
+
+impl RpcService {
+    fn children(&self) -> Vec<DeclarationIndex> {
+        let mut children = BTreeSet::new();
+        for method in self.methods.values() {
+            method.argument_type.kind.add_children(&mut children);
+            method.return_type.kind.add_children(&mut children);
+        }
+        children.into_iter().collect()
+    }
+}
